/*
 * Created on 15-Sep-2003
 *
 */
package pong;

import java.io.Serializable;

/**
 * Classe que representa o estado da bola. 
 * O estado da bola  dado pela sua posio (x,y) e pelo seu vector de velocidade, tambm
 * dado por duas componentes (Vx,Vy).
 * <p> ATENO: No  necessrio modificar estes valores directamente. Isto  feito pela interface
 * que manipula os objectos que se movem no ecran.</p>
 * 
 * @author nuno
 */
public class BallState implements Serializable {
	private int xPosition;
	private int yPosition;
	private int xSpeed;
	private int ySpeed;
	
	private int player;

	/**
	 * Construtor da classe.
	 * @param x posio xx da bola
	 * @param y posio yy da bola
	 * @param xs componente xx do vector velocidade da bola.
	 * @param ys componente yy do vector velocidade da bola.
	 */
	public BallState(int x, int y, int xs, int ys){
		xPosition = x;
		yPosition = y;
		xSpeed = xs;
		ySpeed = ys;
	}

	/**
	 * Devolve o componente xx da Posio actual da bola.
	 * @return o componente xx da posio actual da bola.
	 */
	public int getXPosition() {
		return xPosition;
	}

	/**
	 * Actualiza o componente xx da posio da bola.
	 * @param position o novo valor para xx da posio da bola.
	 */
	public void setXPosition(int position) {
		xPosition = position;
	}

	/**
	 * Devolve o componente xx da Velocidade actual da bola.
	 * @return o componente xx da Velocidade actual da bola.
	 */
	public int getXSpeed() {
		return xSpeed;
	}

	/**
	 *  Actualiza o valor do componente xx do vector velocidade da bola.
	 * @param speed o novo valor para o componente xx do vector velocidade da bola.
	 */
	public void setXSpeed(int speed) {
		xSpeed = speed;
	}

	/**
	 * Devolve o componente yy da Posio actual da bola.
	 * @return o componente yy da Posio actual da bola.
	 */
	public int getYPosition() {
		return yPosition;
	}

	/**
	 * Actualiza o componente yy da posio da bola.
	 * @param position o novo valor para yy da posio da bola.
	 */
	public void setYPosition(int position) {
		yPosition = position;
	}

	/**
	 * Devolve o componente yy da Velocidade actual da bola.
	 * @return o componente yy da Velocidade actual da bola.
	 */
	public int getYSpeed() {
		return ySpeed;
	}

	/**
	 * Actualiza o valor do componente yy do vector velocidade da bola.
	 * @param speed o novo valor para o componente yy do vector velocidade da bola.
	 */
	public void setYSpeed(int speed) {
		ySpeed = speed;
	}

	/**
	 * Incrementa a posio (x,y) da bola, tendo em conta a sua velocidade.
	 */
	protected void incPosition(){
		xPosition += xSpeed;
		yPosition += ySpeed;
	}

}

