package pong;

import java.awt.Graphics;
import java.awt.Point;

import appia.Channel;

/**
 * <b>Classe que cria a interface do jogo e que disponibiliza a API a ser utilizada</b>.
 * Esta classe  a API que permite interagir com a interface grfica. 
 * @author nuno
 */
public class PongInterface {

	private PongGraphics pong;
	private Graphics graphics;
	
	/**
	 * Construtor da classe. 
	 *  criada uma instancia da interface com os "Paddles".
	 * A Bola no  criada ainda. Dever ser criada atravs do mtodo createBall().
	 * @param appiaChannel o Canal Appia para onde so enviados os eventos
	 * gerados pelo movimento do rato.
	 * @param myPaddle dever ser um nmero entre 1 e 4.  o nmero do jogador
	 * (ou o nmero do Paddle que  o mesmo).
	 */
	public PongInterface(Channel appiaChannel, int myPaddle){
		pong = new PongGraphics(appiaChannel, myPaddle);
		graphics = pong.getCanvas().getGraphics();
	}

	/**
	 * Constroi uma nova bola, inicializada numa posio por omisso. 
	 * A bola  nica no jogo. Assim,
	 * se a bola j foi criada, a chamada a esta funo no tem efeito.
	 */
	public void createBall(){
		pong.createBall();
	}

	/**
	 * Constroi uma nova bola e  colocada na posio dada. A bola  nica no jogo. Assim,
	 * se a bola j foi criada, a chamada a esta funo no tem efeito.
	 * @param initialPos posio inicial da bola.
	 */
	public void createBall(Point initialPos){
		pong.createBall(initialPos);
	}

	/**
	 * Move a bola para a posio seguinte. 
	 * So calculados quer a nova posio,
	 * quer o vector de velocidade e a bola  redezenhada no ecran.
	 * Se a bola atingiu uma parede, significa que  "golo" e o nmero do jogador
	 *  retornado pelo mtodo.
	 * Este mtodo dever ser invocado periodicamente para dar o efeito de bola em movimento.
	 * @return o nmero do jogador que sofreu um golo ou -1 se no foi golo.
	 */
	public int moveBall(){
		pong.repaint();
		try {
			pong.getBall().move(graphics);
			return -1;
		} catch (OutOfFieldException e) {
			return e.getWall();
		}
	}

	/**
	 * Move a bola para a posio dada. 
	 * O estado da bola  actualizado e a bola  redezenhada no ecran.
	 * Este mtodo dever ser invocado periodicamente para dar o efeito de bola em movimento.
	 * @param newState o novo estado da bola (posio e vector de velocidade).
	 */
	public void moveBall(BallState newState){
			pong.getBall().move(newState, graphics);
	}

	/**
	 * Move o Paddle para a posio dada.
	 * O Paddle  redezenhado no ecran.
	 * @param player o nmero do jogador do Paddle correspondente, que dever ser entre 1 e 4.
	 * @param point a nova posio do paddle, que  dada pelo movimento do rato.
	 */
	public void movePaddle(int player, Point point){
		pong.getPaddle(player).move(point, graphics);
	}

	/**
	 * Activa o Paddle do jogador correspondente.
	 * Este mtodo faz com que o Paddle
	 * mude de cor, assinalando que o jogador est acivo. Se o jogador no estiver presente
	 * o Paddle est vermelho, caso contrrio est cinzento.
	 * @param player o nmero do jogador correspondente.
	 * @param active verdadeiro ou falso para activar/desactivar o Paddle.
	 */
	public void activePaddle(int player, boolean active){
		pong.getPaddle(player).setActive(active);
	}

	/**
	 * Acrescenta o nmero do Jogador ao ttulo da janela.
	 * @param player nmero do jogador.
	 */
	public void setWindowTitle(int player){
		pong.setWindowTitle(player);
	}

	/**
	 * Actualiza a Label com a pontuao do Jogador.
	 * @param player o jogador que sofreu um golo.
	 * @param newPont nova pontuao do jogador.
	 */
	public void updatePontuation(int player, int newPont){
		pong.updatePontuation(player, newPont);
	}	
	
	/**
	 * Devolve o pontuao de um determinado jogador.
	 * @param player nmero do jogador
	 * @return a pontuao do jogador.
	 */
	public int getPontuation(int player){
		return pong.getPontuation(player);
	}
	
	/**
	 * Devolve o estado actual da bola. 
	 * Os mtodos desta classe no deveram ser usados directamente. 
	 * Esta classe deve ser usada para visualizar o estado da bola e, se necessrio,
	 * partilh-lo com outros processos.
	 * @return o estado actual da bola.
	 */
	public BallState getBallState(){
		if(pong.getBall() == null)
			return null;
		return pong.getBall().getBallState();
	}
}
