/*
 * Created on 15-Sep-2003
 *
 */
package test;

import java.awt.Point;
import java.net.InetAddress;
import java.net.UnknownHostException;

import pong.PongInterface;
import pong.events.ButtonStartEvent;
import pong.events.MouseMovedEvent;
import pong.events.MoveBallTimer;
import appia.AppiaEventException;
import appia.AppiaException;
import appia.Channel;
import appia.Direction;
import appia.Event;
import appia.EventQualifier;
import appia.Layer;
import appia.Session;
import appia.events.channel.ChannelInit;
import appia.protocols.group.events.ObjectsMessage;
import appia.protocols.udpsimple.InetWithPort;
import appia.protocols.udpsimple.RegisterSocketEvent;

/**
 * @author nuno
 */
public class ApplSession extends Session {

	private Channel channel;
	private PongInterface pong;
	private InetWithPort local, remote;
	private int activePlayers, myPaddle;

	/**
	 * Construtor da classe.
	 * @param layer
	 */
	public ApplSession(Layer layer) {
		super(layer);
		activePlayers = 0;
	}

	/*
	 * Metodo invocado antes de se inicializar o Canal Appia.
	 */
	public void init(int localPort, InetWithPort remote, int paddle){
		try {
			local = new InetWithPort(InetAddress.getLocalHost(),localPort);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} 
		this.remote = remote;
		myPaddle = paddle;
	}

	/**
	 * Metodo chamado pelo sistema Appia para entreguar um evento a esta camada.
	 * A Thread do Appia nunca deve ser bloqueada.
	 * Quando o Appia invoca o handle() de cada camada, esta deve
	 * sempre executar algum processamento e retornar.
	 */
	public void handle(Event ev) {
		//System.out.println("ApplSession Recebeu evento "+ev.getClass().getName());
		if (ev instanceof ChannelInit)
			handleChannelInit((ChannelInit) ev);
		else if (ev instanceof MouseMovedEvent)
			handleMouseMovedEvent((MouseMovedEvent) ev);
		else if (ev instanceof ButtonStartEvent)
			handleButtonStartEvent((ButtonStartEvent) ev);
		else if (ev instanceof MoveBallTimer)
			handleMoveBallTimer((MoveBallTimer) ev);
		else
			// evento inesperado. o metodo go() serve para reencaminhar os eventos.
			try {
				ev.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}

	}

	/**
	 * @param init
	 */
	private void handleChannelInit(ChannelInit init) {
		/*
		 * este  o primeiro evento a aparecer em qualquer camada.
		 * dever ser tambm o primeiro a ser reencaminhado.
		 * Nunca se deve enviar um evento antes de reencaminhar este.
		 */
		
		channel = init.getChannel();
		try {
			init.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
		/*
		 * Criao da Interface Grfica.
		 * Recebe o canal Appia e o nmero do Paddle local.
		 */
		pong = new PongInterface(channel, myPaddle);
		/*
		 * Este evento serve para registar um socket na camada usada como
		 * interface entre o Appia e os sockets.
		 */
		try {
			RegisterSocketEvent rse = new RegisterSocketEvent(channel,Direction.DOWN,this,local.port);
			rse.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		}
	}

	/**
	 * @param timer
	 */
	private void handleMoveBallTimer(MoveBallTimer timer) {
		/*
		 * Este timer aparece periodicamente (depois de ser inicializado)
		 * e  aqui que  tratado.
		 */
		int player = pong.moveBall();
		if (player != -1){
			// Se o resultado de "moveBall()" for diferente de -1,  porque bateu
			// na parede de algum. Sendo assim, incrementa-se a pontuao desse jogador.
			int newPontuation = pong.getPontuation(player)+1;
			pong.updatePontuation(player,newPontuation);
		}
		try {
			timer.go();
		} catch (AppiaEventException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param event
	 */
	private void handleMouseMovedEvent(MouseMovedEvent event) {
		/*
		 * Este evento aparece sempre que algum mexe o rato em cima do campo.
		 */
		ObjectsMessage message = event.getObjectsMessage();
		/*
		 *  atravs da sua direco que se percebe de onde vem o evento.
		 * Evidentemente, se for para baixo  porque vem da aplicao,
		 * seno  porque vem da rede. 
		 */
		if(event.getDir() == Direction.DOWN){
			pong.movePaddle(event.getPaddle(), event.getPoint());
			/*
			 * Estes dois campos so preenchidos com um Objecto
			 * do tipo InetWithPort s quando no se usa sincronia virtual.
			 * As camadas acima das camadas de sincronia virtual usam um rank. 
			 * Ver o JavaDoc do Appia em http://appia.di.fc.ul.pt/online_docs/javadoc/
			 * para mais detalhes sobre o GroupSendableEvent.
			 */
			event.source = local;
			event.dest = remote;
			/*
			 * A nica coisa que passa pela rede  o contedo da mensagem que o evento contm,
			 * neste caso, uma coisa do tipo ObjectsMessage. Se quisermos que os atributos do
			 * evento por ns criado sejam passados pela rede, temos de os colocar na mensagem.
			 */
			message.pushObject(event.getPoint());
			message.pushInt(event.getPaddle());
			try {
				/*
				 * O atributo source do evento  diferente do metodo set/getSource()
				 * Este mtodo devolve/actualiza a sesso onde foi criado o evento.
				 * Se for um evento que nos foi entregue, basta reeencaminha-lo sem problemas.
				 * Se for um evento por ns criado, um evento clonado ou um evento que surgiu
				 * fora do Canal Appia (que  este o caso) deve-se fazer setSource(this)
				 * 
				 * Quando se modifica este atributo (ou a direco, ou o Canal) deve-se fazer init()
				 * ao evento. 
				 */
				event.setSource(this);
				event.init();
				event.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
		else {
			/*
			 * A mensagem de um evento funciona como uma pilha.
			 * Assim, tem de se fazer pop() (tirar a informao) no sentido contrrio
			 * em que se fez push() (colocar a informao).
			 */
			int paddle = message.popInt();
			Point point = (Point) message.popObject();
			event.setPoint(point);
			event.setPaddle(paddle);
			pong.movePaddle(event.getPaddle(), event.getPoint());
		}
	}

	/**
	 * @param event
	 */
	private void handleButtonStartEvent(ButtonStartEvent event) {
		ObjectsMessage message = event.getObjectsMessage();
		if(event.getDir() == Direction.DOWN){
			pong.activePaddle(event.getPaddle(), true);
			pong.setWindowTitle(event.getPaddle());
			event.source = local;
			event.dest = remote;
			message.pushInt(event.getPaddle());
			try {
				event.setSource(this);
				event.init();
				event.go();
			} catch (AppiaEventException e) {
				e.printStackTrace();
			}
		}
		else {
			int paddle = message.popInt();
			event.setPaddle(paddle);
			pong.activePaddle(event.getPaddle(), true);
		}


		/*
		 * Soluo "bacoca" para verificar se os dois elementos j se inicializaram :-)
		 * Vos teram de arranjar outra ;)
		 * --> IMPORTANTE: Ver startBall()
		 */
		activePlayers++;
		if(activePlayers==2)
			startBall();
	}

	private void startBall(){
		/*
		 *  atravs deste mtodo que a bola  criada na interface.
		 */
		pong.createBall();
		try {
			/*
			 * este pedao de cdigo mostra como se inicializa um timer periodico.
			 * para cancelar o timer, deve-se fazer o mesmo, mas com o EventQualifier.OFF 
			 */
			MoveBallTimer timer =
				new MoveBallTimer(100,channel,this, EventQualifier.ON);
			timer.go();
		} catch (AppiaEventException e1) {
			e1.printStackTrace();
		} catch (AppiaException e1) {
			e1.printStackTrace();
		}
	}

}
